"""
Data decorator. Originally created to be used in every method of the model class
it will convert the method to a datapoint/node that can be set and reset.

Next feature: It will allow using a reference datapoints automatically when sent in
the "ref" parameter of a Flet Control.
"""

__all__ = [
    'data'
]

class _Null:
    """ Null class that will be used when the datapoint is still empty """
    def __bool__(self):
        return False

    def __repr__(self):
        return 'Null'

NULL = _Null()


class data:
    """
    data decorator
    Converts a class method (intended for the Model class) to a datapoint
    """
    def __init__(self, f):
        self.func = f
        self.current_value = NULL
    
    
    def __call__(self):
        """
        Return current value when datapoint is called
        e.g. print(self.model.fruit()) -> 'Apple'
        """
        return self.current_value
        
    def __get__(self, instance, owner):
        """
        Return value when accessing the datapoint instance.
        This will return the self instance of the datapoint in order to
        access the other methods of the datapoint.
        """
        self.model_instance = instance
        self.model_class = self.model_instance.__class__
        
        if self.current_value is NULL:
            self.current_value = self.func(instance)
            
        return self
        
    def set_value(self, x):
        """
        Set value method
        e.g.
        self.model.fruit.set_value('Pear')
        print(model.fruit()) -> 'Pear'
        """
        self.current_value = x
    
    def reset(self):
        """
        Reset methods - Resets a datapoint to their initial value
        e.g.
        print(self.model.fruit()) -> 'Apple'
        self.model.fruit.set_value('Pear')
        print(self.model.fruit()) -> 'Pear'
        self.model.fruit.reset()
        print(model.fruit()) -> 'Apple'
        """
        self.current_value = self.func(self.model_instance)

    # TODO: Here the current setter and getter will be defined in order to save a ref object in a datapoint     
    
    # Magic Methods so the datapoint can be used as any other data type
    def __lt__(self, other):
        return self.current_value < other
    
    def __le__(self, other):
        return self.current_value <= other
    
    def __eq__(self, other):
        return self.current_value == other
    
    def __ne__(self, other):
        return self.current_value != other
    
    def __gt__(self, other):
        return self.current_value > other
    
    def __ge__(self, other):
        return self.current_value >= other
    
    def __len__(self):
        return len(self.current_value)
    
    def __add__(self, other):
        return self.current_value + other
    
    def __sub__(self, other):
        return self.current_value - other
    
    def __div__(self, other):
        return self.current_value/other
    
    def __mul__(self, other):
        return self.current_value * other
    
    def __neg__(self):
        return -self.current_value
    
    def __pos__(self):
        return +self.current_value
    
    def __abs__(self):
        return abs(self.current_value)
    
    def __invert__(self):
        return ~self.current_value


from setuptools import setup, find_packages
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

VERSION = '0.0.4'
DESCRIPTION = 'This package will allow the developer to use an mvc structure in a Flet project.'

# Setting up
setup(
    name="flet-mvc",
    version=VERSION,
    author="o0Adrian (C. Adrián Monroy)",
    author_email="<carlos.monclef@gmail.com>",
    description=DESCRIPTION,
    url='https://github.com/o0Adrian/flet-mvc',
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=['flet'],
    keywords=['mvc', 'flet', 'flet mvc', 'model', 'view', 'controller', 'node', 'datapoint'],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)


"""
Flet Base View Class
"""
from .controller import FletController

class FletView():
    def __init__(self, model = None, content = [], controller: FletController = None):
        self.controller = controller
        self.model = model
        self.content = content


"""
Flet Base Controller Class
"""

import flet as ft
from .alert import ERROR, WARNING, SUCCESS, ADVICE, INFO

class FletController():
    def __init__(self, page: ft.Page, model):
        self.page = page
        self.model = model
        
    def update(self):
        """ Updates the page by using self.update() instead of self.page.update() """
        self.page.update()

        
    def alert(self, msg="" , alert_type=ERROR):
        """ Show Alert """
        if alert_type == ERROR:
            self.page.snack_bar = ft.SnackBar(
                content=ft.Row(
                    controls=[
                        ft.Icon(ft.icons.ERROR_OUTLINE, color=ft.colors.WHITE),
                        ft.Text(msg),
                    ],
                ),
                bgcolor=ft.colors.ERROR,
            )
            self.page.snack_bar.open = True
            self.update()
            
        elif alert_type == SUCCESS:
            self.page.snack_bar = ft.SnackBar(
                content=ft.Row(
                    controls=[
                        ft.Icon(ft.icons.CHECK_CIRCLE, color=ft.colors.WHITE),
                        ft.Text(msg),
                    ],
                ),
                action="OK",
                action_color=ft.colors.BLACK,
                bgcolor=ft.colors.GREEN_300,
            )
            self.page.snack_bar.open = True
            self.update()
            
        elif alert_type == ADVICE:
            self.page.snack_bar = ft.SnackBar(
                    content=ft.Text(msg),
                    action="Understood",
                    action_color=ft.colors.BLUE_200
                )
            self.page.snack_bar.open = True
            self.update()
            
        elif alert_type == INFO:
            self.page.snack_bar = ft.SnackBar(
                    content=ft.Row(
                        controls=[
                            ft.Icon(ft.icons.INFO_OUTLINED, color=ft.colors.WHITE),
                            ft.Text(msg),
                        ],
                    ),
                    bgcolor=ft.colors.BLUE_200
            )
            self.page.snack_bar.open = True
            self.update()
            
        elif alert_type == WARNING:
            self.page.snack_bar = ft.SnackBar(
                content=ft.Row(
                    controls=[
                        ft.Icon(ft.icons.WARNING_AMBER_ROUNDED, color=ft.colors.WHITE),
                        ft.Text(msg),
                    ],
                ),
                action="OK",
                action_color=ft.colors.BLACK,
                bgcolor=ft.colors.AMBER,
            )
            self.page.snack_bar.open = True
            self.update()
